function [next_state_num, act, Qt] = sw_move_state(w12, b2, w23, b3, ...
    HiddenFunc, OutFunc, epsilon, randdata, current_state_num, maze_i, maze_j)
% ACTION : Get action, next state, and Q values.
%
% Syntax : [next_state_num, act, Qt] = SW_MOVE_STATE(w12, b2, w23, b3, ...
%            HiddenFunc, OutFunc, epsilon, randdata, current_state_num, maze_i, maze_j);
%
% Input  :  w12    (Required) : Weight (Between input layer and hidden layer)
%                               (9xNum. hidden units  Matrix)
%           b2     (Required) : Baias of hidden layer.
%                               (Num. hidden units x 1  Vector)
%           w23    (Required) : Weight (Between hidden layer and output layer)
%                               (Num. hidden units x 5  Matrix)
%           b3     (Required) : Baias of output layer.
%                               (4 x 1  Vector)
%           HiddenFunc (Required) : Definition of hidden function.
%                               (Char : 'Sigmoid')
%           OutFunc    (Required) : Definition of output function.
%                                 : (Char : 'SoftMax')
%           epsilon    (Required) : Epsilon. (Scalar : 0 to MaxEpisode)
%           randdata   (Required) : Random data (Scalar : 0 to MaxEpisode)
%           satate_num (Required) : State number (Scalar : 1 to Size of Maze (Ex. 3x3))
%           maze_i     (Required) : Size of Mase (i(y) axis : c)
%           maze_j     (Required) : Size of Mase (j(x) axis : )
%
% Output :  new_satate_num : Next state number (Scalar : 1 to Size of Maze (Ex. 3x3))
%           act : Action. (Scalar : 1 to 4)
%           Qt : Output of neural network. (4x1 Vector)
%
% Requirements : None.
%
% See also SW_Q_LEARNING.    

    % Ԃ One hot ͂̍쐬
    st = zeros(maze_i*maze_j,1);       % Ԃ One hot ͂̍쐬
    st(current_state_num) = 1;

    X = st;                             % Input (State Number)
    
    z2 = w12.' * X + b2;                % Bw̏dݕtaz2
    if(HiddenFunc == "sigmoid")
        a2 = 1./(1+exp(-z2));           % Bw̏oa2i֐FVOCh֐j
    end
    
    z3 = w23.' * a2 + b3;               % o͑w̏dݕtaz3
    if(OutFunc == 'Linear')
        a3 = z3;    
    end
    Qt = a3;                            % o͂QtƂĊi[
    
    % l̊mF
    % fprintf('\nQt =');
    % fprintf('%f ',Qt);
    % fprintf('\n');
    
    % si-greedy ASYj
    if epsilon <= randdata              % epsilon胉_f[^΃_ōs
        max_value = max(Qt);            % Ql̍őlpčs
        for j = 1 : 4
            if max_value == Qt(j)
                act = j ;               % ől̃CfbNXsl
            end
        end
    else
        % 25p[Zg4̍siCCCjI
%         fprintf('RA'); % Display randum action.
        random_value = rand;
        if random_value <= 0.25
            act = 1 ; % 4̍sij
        elseif random_value <= 0.5
            act = 2 ; % 4̍sij
        elseif random_value <= 0.75
            act = 3 ; % 4̍sij
        else
            act = 4 ; % 4̍sij
        end
    end
    
    % JڐɈڂHՖʂ̍쐬
    if act == 1            % 4̍sij
        next_state_num = current_state_num + 1;
        % sʁCՖʂ̘gOɍsꍇ́C̏ŗ܂D
        if next_state_num > maze_i*maze_j || rem(next_state_num, maze_j) == 1
            next_state_num = current_state_num;
        end        
    elseif act == 2        % 4̍sij
        next_state_num = current_state_num - maze_j;
        % sʁCՖʂ̘gOɍsꍇ́C̏ŗ܂D
        if next_state_num <= 0
            next_state_num = current_state_num;
        end
    elseif act == 3        % 4̍sij
        next_state_num = current_state_num - 1;
        % sʁCՖʂ̘gOɍsꍇ́C̏ŗ܂D
        if next_state_num <= 0 || rem(next_state_num, maze_j) == 0
            next_state_num = current_state_num;
        end
    else                   % 4̍sij
        next_state_num = current_state_num + maze_j;
        % sʁCՖʂ̘gOɍsꍇ́C̏ŗ܂D
        if next_state_num > maze_i*maze_j
            next_state_num = current_state_num;
        end
    end        
end