function [w12, b2, w23, b3] = sw_update_Q(current_state_num, next_state_num, ...
    w12, b2, w23, b3, HiddenFunc, OutFunc, at, reward, penalty, alpha, ganma, maze_i, maze_j)
% SW_UPDATE_Q : Update weight and bias vlues.
%
% Syntax : [w12, b2, w23, b3] = SW_UPDATE_QVALUE(current_state_num, next_state_num, ...
%                                                w12, b2, w23, b3, HiddenFanc, OutFunc, at, ...
%                                                reward, penalty, alpha, ganma, maze_i, maze_j);
%
% Input  :  current_state_num (Requred) : Current state (state_t) information. (Scalar : 1 to 9)
%           next_state_num (Requred) : Next state (state_t+1) information. (Scalar : 1 to 9)
%           w12     (Required) : Weight (Between input layer and hidden layer)
%                               (9xNum. hidden units  Matrix)
%           b2      (Required) : Baias of hidden layer.
%                               (Num. hidden units x 1  Vector)
%           w23     (Required) : Weight (Between hidden layer and output layer)
%                               (Num. hidden units x 5  Matrix)
%           b3      (Required) : Baias of output layer.
%                               (4 x 1  Vector)
%           HiddenFunc (Required) : Definition of hidden function.
%                               (Char : 'Sigmoid')
%           OutFunc (Required) : Definition of output function.
%                               (Char : 'SoftMax')
%           at      (Required) : Current action (action_t). (Scalar : 1 to 4)
%           reward  (Required) : Reward (V).  (Scalar : Integer)
%           penalty (Required) : Penalty (yieB).  (Scalar : Integer)
%           alpha   (Required) : Leaning rate (wK).  (Scalar : 0 to 1.0)
%           ganma   (Required) : Discount factor (). (Scalar : 0 to 1.0)
%           maze_i  (Required) : Size of Mase (i(y) axis : c)
%           maze_j  (Required) : Size of Mase (j(x) axis : )
%
% Output :  w12     (Required) : Weight (Between input layer and hidden layer)
%                               (9xNum. hidden units  Matrix)
%           b2      (Required) : Baias of hidden layer.
%                               (Num. hidden units x 1  Vector)
%           w23     (Required) : Weight (Between hidden layer and output layer)
%                               (Num. hidden units x 5  Matrix)
%           b3      (Required) : Baias of output layer.
%                               (4 x 1  Vector)
%
% See also SW_Q_LEARNING.

    % VmFiIɏ㏑j
    %     if next_state_num == 9      % S[
    %         reward = 10 ;
    %     elseif t == 15          % 15sĂS[łȂΔ
    %         reward = 0 ;
    %     end
    
    % ͎̏ԂQlisplǰvZ
    st1 = zeros(maze_i*maze_j, 1);      % One hot ͂̍쐬
    st1(next_state_num) = 1;

    % Next State : Input (State Number)
    X = st1;
    
    z2 = w12.' * X + b2;                % Bw̏dݕtaz2
    
    if(HiddenFunc == "sigmoid")
        a2 = 1./(1+exp(-z2));           % Bw̏oa3i֐FVOCh֐j
    end
    
    z3 = w23.' * a2 + b3;               % o͑w̏dݕtaz3
    
    if(OutFunc == 'Linear')
        a3 = z3;        
    end
    Qt1 = a3;                           % ̏Ԃ̏o͂Qt1ƂĊi[
    
    % ͍̏ԂQlisplǰvZ
    % Current State : Input (State Number)
    st = zeros(maze_i*maze_j, 1);
    st(current_state_num) = 1;
    
    X = st;
    
    z2 = w12.' * X + b2;                % Bw̏dݕtaz2
    if(HiddenFunc == "sigmoid")
        a2 = 1./(1+exp(-z2));               % Bw̏oa2i֐FVOCh֐j
        da2dz2 = (1-a2).*a2;                % a^(2)/z^(2) = (1-a^(2))*a^(2)
    end
    
    z3 = w23.' * a2 + b3;               % o͑w̏dݕtaz3
    if(OutFunc == 'Linear')
        a3 = z3;
        da3dz3 = 1;
    end
    Qt = a3;                            % ̏Ԃ̏o͂Qt1ƂĊi[
    
    % 덷`i2덷ƂĂj
    Et = (reward - penalty + ganma * max(Qt1) - Qt(at)).^2;
    Ct = zeros(4,1);
    Ct(at) = Ct(at) + (reward - penalty + ganma * max(Qt1) - Qt(at)).^2;
    %
    % 덷̊mF
    % fprintf('Ct = ');
    % fprintf('%f ',Ct);
    % fprintf('\n');
    
    % NNXV iXCh7jKPɋL
    dCda3 = zeros(4,1);
    dCda3(at) = dCda3(at) + (reward - penalty + ganma * max(Qt1) - Qt(at));
                                        % 2덷ł邽߁CĂ͕̎ωȂD

    dCdz3 = dCda3 .* da3dz3;            % C/z^(3) = C/a^(3) x a^(3)/z^(3)
    delta3 = dCdz3;                     % ^(3) = C/z^(3)
    
    dCdw23 = a2 * delta3.';             % C/w^(3) = C/z^(3) x z^(3)/w^(3) =  ^(3) a^(2)
    
    dCda2 = w23 * delta3;               % C/a^(2) =  C/z^(3) x z^(3/a^(2) 
                                        %             = ^(3) w^(3) 
                                        %             = w^(3) * ^(3)
                                        %             (s̊|ZŉZ\j
    dCdz2 = dCda2 .* da2dz2;            % C/z^(2) = C/a^(2) x a^(2)/z^(2)
    delta2 = dCdz2;                     % ^(2) = C/z^(2)
    
    dCdw12 = X * delta2.';              % C/w^(2) = X ^(2)       (X = a^(1))

    w23 = w23 + alpha * dCdw23;
    b3  = b3  + alpha * delta3;
    w12 = w12 + alpha * dCdw12;
    b2  = b2  + alpha * delta2;
    
end